/*!
 * \file ci.c
 *
 * \author FTDI
 * \date 20120114
 *
 * Copyright  2011 Future Technology Devices International Limited
 * Company Confidential
 *
 * Project: FT12x
 * Module: LPC1114-FT12x Eval
 *
 * Rivision History:
 * 0.1 - initial version
 */

#include "..//include//al.h"
#include "..//include//mainloop.h"
#include "..//include//ci.h"
#include "..//include//ftdi.h"

extern EPPFLAGS bEPPflags;
uint8_t inBuf[10], outBuf[10];

void FT12x_SetAddressEnable(unsigned char bAddress, unsigned char bEnable)
{
	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	if(bEnable)
		bAddress |= 0x80;
	
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=0xD0;
		outBuf[1]=bAddress;
		SSP_Send(SSP_NUM,outBuf,2);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0xD0);
		outportb(FT12X_DATA, bAddress);
	}
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
}

void FT12x_SetEndpointEnable(unsigned char bEnable)
{
	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=0xD8;
		if(bEnable)
			outBuf[1]=1;
		else
			outBuf[1]=0;
		SSP_Send(SSP_NUM,outBuf,2);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0xD8);
		if(bEnable)
			outportb(FT12X_DATA, 1);
		else
			outportb(FT12X_DATA, 0);
	}
	
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
}

void FT12x_SetMode(unsigned char bConfig, unsigned char bClkDiv)
{
	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=0xF3;
		outBuf[1]=bConfig;
		outBuf[2]=bClkDiv;
		SSP_Send(SSP_NUM,outBuf,3);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0xF3);
		outportb(FT12X_DATA, bConfig);
		outportb(FT12X_DATA, bClkDiv);
	}
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
}

void FT12x_SetDMA(unsigned char bMode)
{

	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=0xFB;
		outBuf[1]=bMode;
		SSP_Send(SSP_NUM,outBuf,2);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0xFB);
		outportb(FT12X_DATA, bMode);
	}

	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
}

unsigned int FT12x_ReadInterruptRegister(void)
{
	unsigned int j=0;
	unsigned char b1;
	
	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
#ifdef ENABLE_ENHANCED_MODE
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=0xF4;
		SSP_Send(SSP_NUM,outBuf,1);
		SSP_Receive(SSP_NUM,inBuf,4);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0xF4);
		inBuf[0] = inportb(FT12X_DATA);
		inBuf[1] = inportb(FT12X_DATA);
		inBuf[2] = inportb(FT12X_DATA);
		inBuf[3] = inportb(FT12X_DATA);
	}
	j = inBuf[3];
	j <<= 8;
	j += inBuf[2];
	j <<= 8;
	j += inBuf[1];
	j <<= 8;
	j += inBuf[0];
#else
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=0xF4;
		SSP_Send(SSP_NUM,outBuf,1);
		SSP_Receive(SSP_NUM,inBuf,2);
		b1=(unsigned char)inBuf[0];
		j=(unsigned int)inBuf[1];
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0xF4);
		b1 = inportb(FT12X_DATA);
		j = inportb(FT12X_DATA);
	}
	j <<= 8;
	j += b1;

	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
#endif
	return j;
}

unsigned char FT12x_SelectEndpoint(unsigned char bEndp)
{
	unsigned char c;

	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=bEndp;
		SSP_Send(SSP_NUM,outBuf,1);
		SSP_Receive(SSP_NUM,inBuf,1);
		c = (unsigned char) inBuf[0];
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, bEndp);
		c = inportb(FT12X_DATA);
	}
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;

	return c;
}

unsigned char FT12x_ReadLastTransactionStatus(unsigned char bEndp)
{
	unsigned char c;
	
	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= 0x40 + bEndp;
		SSP_Send(SSP_NUM,outBuf,1);
		SSP_Receive(SSP_NUM,inBuf,1);
		c = (unsigned char) inBuf[0];
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0x40 + bEndp);
		c = inportb(FT12X_DATA);
	}
	
	if(bEPPflags.bits.in_isr == 0)
			ENABLE;
	return c;
}

unsigned char FT12x_ReadEndpointStatus(unsigned char bEndp)
{
	unsigned char c;

	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= 0x80 + bEndp;
		SSP_Send(SSP_NUM,outBuf,1);
		SSP_Receive(SSP_NUM,inBuf,1);
		c = (unsigned char) inBuf[0];
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0x80 + bEndp);
		c = inportb(FT12X_DATA);
	}
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;

	return c;
}

void FT12x_SetEndpointStatus(unsigned char bEndp, unsigned char bStalled)
{
	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	if(isSpi)
	{
		//SetEndpointStatus command is 0x50 for FT121; and 0x40 for FT120/FT122
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= 0x50 + bEndp;
		outBuf[1]= bStalled;
		SSP_Send(SSP_NUM,outBuf,2);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0x40 + bEndp);
		outportb(FT12X_DATA, bStalled);
	}
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
}

void FT12x_SendResume(void)
{
	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= 0xF6;
		SSP_Send(SSP_NUM,outBuf,1);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
		outportb(FT12X_COMMAND, 0xF6);
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
}

unsigned short FT12x_ReadCurrentFrameNumber(void)
{
	unsigned short i,j;

	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= 0xF5;
		SSP_Send(SSP_NUM,outBuf,1);
		SSP_Receive(SSP_NUM,inBuf,2);
		i = (unsigned short) inBuf[0];
		j = (unsigned short) inBuf[1];
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0xF5);
		i= inportb(FT12X_DATA);
		j = inportb(FT12X_DATA);
	}
	i += (j<<8);

	if(bEPPflags.bits.in_isr == 0)
		ENABLE;

	return i;
}

unsigned short FT12x_ReadChipID(void)
{
	unsigned short i,j;

	if(bEPPflags.bits.in_isr == 0)
		DISABLE;

	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=0xFD;
		SSP_Send(SSP_NUM,outBuf,1);//command 0xFD
		SSP_Receive(SSP_NUM,inBuf,2);//read 2byte ChipID
		i = inBuf[1];
		i = i<<8;
		i += inBuf[0];
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, 0xFD);
		i=inportb(FT12X_DATA);
		j=inportb(FT12X_DATA);
		i += (j<<8);
	}

	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
	return i;
}

unsigned char FT12x_ReadEndpoint(unsigned char endp, unsigned char * buf, unsigned char len)
{
	unsigned char i, j, k;

	DBG(MSG_DEBUG, "ENTER FT12x_ReadEndpoint EP=%u len=%u\r\n",(unsigned)endp,(unsigned)len);

	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= endp;
		SSP_Send(SSP_NUM,outBuf,1);
		SSP_Receive(SSP_NUM,inBuf,1);
		GPIOSetValue( 2, 0, 1 );//SS high

		if((inBuf[0] & FT12X_FULLEMPTY)==0)
		{
			if(bEPPflags.bits.in_isr == 0)
				ENABLE;
			return 0;
		}

		GPIOSetValue( 2, 0, 0 );//SS low
		//ReadBuffer command is 0xE0 for FT121; and 0xF0(same as WriteBuffer) for FT120, FT122
		outBuf[0]= 0xE0;
		SSP_Send(SSP_NUM,outBuf,1);

		SSP_Receive(SSP_NUM,tempBuffer,len+2);
		for(k=0;k<tempBuffer[1];k++)
			buf[k]=tempBuffer[k+2];
		i=tempBuffer[0];
		j=tempBuffer[1];
		if(j > len)
			j = len;

		GPIOSetValue( 2, 0, 1 );//SS high

		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= 0xF2;
		SSP_Send(SSP_NUM,outBuf,1);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, endp);
		if((inportb(FT12X_DATA) & FT12X_FULLEMPTY) == 0) 
		{
			if(bEPPflags.bits.in_isr == 0)
				ENABLE;
			return 0;
		}
		
		outportb(FT12X_COMMAND, 0xF0);
		i = inportb(FT12X_DATA);
		j = inportb(FT12X_DATA);
		if(j > len)
			j = len;
		
		#ifdef OPTMISE_FOR_THROUPUT
			FT12x_ReadBuffer((bool)FT12X_DATA, (uint8*)buf, (uint32) j);
		#else

			for(i=0; i<j; i++)
				*(buf+i) = inportb(FT12X_DATA);
		#endif
			outportb(FT12X_COMMAND, 0xF2);
	}

	if(bEPPflags.bits.in_isr == 0)
		ENABLE;

	DBG(MSG_DEBUG, "DATA:");
	for(k=0;k<(j+2);k++)
	{
		DBG(MSG_DEBUG, " 0x%x",buf[k]);
	}

	DBG(MSG_DEBUG, "\r\nEXIT FT12x_ReadEndpoint bytesRead(i=%u, j=%u)\r\n",(unsigned)i, (unsigned)j);

	return j;
}

unsigned char FT12x_ReadMainEndpoint(unsigned char * buf)
{
	unsigned char i, j, k = 0, bDblBuf = 1;

	if(bEPPflags.bits.in_isr == 0)
		DISABLE;

	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= 0x84;
		SSP_Send(SSP_NUM,outBuf,1);
		SSP_Receive(SSP_NUM,inBuf,1);
		GPIOSetValue( 2, 0, 1 );//SS high
		
		if( (inBuf[0] & 0x60) == 0x60)
			bDblBuf = 2;

		while(bDblBuf) 
		{		
			GPIOSetValue( 2, 0, 0 );//SS low
			outBuf[0]= 4;
			SSP_Send(SSP_NUM,outBuf,1);
			SSP_Receive(SSP_NUM,inBuf,1);
			GPIOSetValue( 2, 0, 1 );//SS high

			if((inBuf[0] & FT12X_FULLEMPTY) == 0)
				break;

			GPIOSetValue( 2, 0, 0 );//SS low
			//ReadBuffer command is 0xE0 for FT121; and 0xF0(same as WriteBuffer) for FT120, FT122
			outBuf[0]= 0xE0;
			SSP_Send(SSP_NUM,outBuf,1);
			SSP_Receive(SSP_NUM,inBuf,2);
			j = inBuf[1];
			SSP_Receive(SSP_NUM,buf,j);
			GPIOSetValue( 2, 0, 1 );//SS high

			GPIOSetValue( 2, 0, 0 );//SS low
			outBuf[0]= 0xF2;
			SSP_Send(SSP_NUM,outBuf,1);
			GPIOSetValue( 2, 0, 1 );//SS high
			
			k += j;
			bDblBuf --;
		}
	}
	else
	{
		outportb(FT12X_COMMAND, 0x84);
		if( (inportb(FT12X_DATA) & 0x60) == 0x60)
			bDblBuf = 2;

		while(bDblBuf) 
		{
			outportb(FT12X_COMMAND, 4);
			if((inportb(FT12X_DATA) & FT12X_FULLEMPTY) == 0)
				break;

			outportb(FT12X_COMMAND, 0xF0);
			j = inportb(FT12X_DATA);
			j = inportb(FT12X_DATA);

		for(i=0; i<j; i++)
			*(buf+i+k) = inportb(FT12X_DATA);

			k += j;

			outportb(FT12X_COMMAND, 0xF2);

			bDblBuf --;
		}
	}

	if(bEPPflags.bits.in_isr == 0)
		ENABLE;

	return k;
}

unsigned char FT12x_WriteEndpoint(unsigned char endp, unsigned char * buf, unsigned char len)
{
	unsigned char i;

	DBG(MSG_DEBUG, "ENTER FT12x_WriteEndpoint EP=%u len=%u\r\nDATA:",(unsigned)endp,(unsigned)len);
	for(i=0;i<len;i++)
	{
		DBG(MSG_DEBUG, " 0x%x",buf[i]);
	}
	DBG(MSG_DEBUG, "\r\n");

	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	
	if(isSpi)
	{

		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=endp;
		SSP_Send(SSP_NUM,outBuf,1);
		GPIOSetValue( 2, 0, 1 );//SS high

		FT12x_Delay(10);

		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=0xF0;
		outBuf[1]=0;
		outBuf[2]=len;
		SSP_Send(SSP_NUM,outBuf,3);
		SSP_Send(SSP_NUM,buf,(uint32)len);
		GPIOSetValue( 2, 0, 1 );//SS high

		FT12x_Delay(10);

		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=0xFA;
		SSP_Send(SSP_NUM,outBuf,1);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, endp);
		inportb(FT12X_DATA);

		outportb(FT12X_COMMAND, 0xF0);
		outportb(FT12X_DATA, 0);
		outportb(FT12X_DATA, len);

		#ifdef OPTMISE_FOR_THROUPUT
			FT12x_WriteBuffer((bool)FT12X_DATA, (uint8*)buf, (uint32) len);
		#else
			for(i=0; i<len; i++)
				outportb(FT12X_DATA, *(buf+i));
		#endif
		
		outportb(FT12X_COMMAND, 0xFA);
	}
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;

	return len;
}

void FT12x_AcknowledgeEndpoint(unsigned char endp)
{
	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= endp;
		SSP_Send(SSP_NUM,outBuf,1);
		GPIOSetValue( 2, 0, 1 );//SS high

		GPIOSetValue( 2, 0, 0 );//SS low	
		outBuf[0]= 0xF1;
		SSP_Send(SSP_NUM,outBuf,1);
		GPIOSetValue( 2, 0, 1 );//SS high
		
		if(endp == 0)
		{
			GPIOSetValue( 2, 0, 0 );//SS low
			outBuf[0]= 0xF2;
			SSP_Send(SSP_NUM,outBuf,1);
			GPIOSetValue( 2, 0, 1 );//SS high
		}
	}
	else
	{
		outportb(FT12X_COMMAND, endp);
		outportb(FT12X_COMMAND, 0xF1);
		if(endp == 0)
			outportb(FT12X_COMMAND, 0xF2);
	}
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
}

uint16 FT12x_GetId(uint8 id)
{
	if(bEPPflags.bits.in_isr == 0)
			DISABLE;

	uint16 pid;
	unsigned char i,j;
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]=id;
		SSP_Send(SSP_NUM,outBuf,1);
		SSP_Receive(SSP_NUM,inBuf,2);
		GPIOSetValue( 2, 0, 1 );//SS high
		pid = (uint16) inBuf[1] ;
		pid = pid<<8;
		pid = pid | inBuf[0];
	}
	else
	{
		outportb(FT12X_COMMAND, id);//Read FTDI ID
		j = inportb(FT12X_DATA);
		i = inportb(FT12X_DATA);

		pid = (((uint16)i)<<8) + j;
	}
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;
	
	return pid;
}

void FT12x_SetEndpointConfig(unsigned char bEndp,unsigned char bEnable, unsigned char type, unsigned char size)
{
	unsigned char command, argument;

	command = 0xB0;
	command |= bEndp;
	argument = 0;
	if(bEnable)
		argument |= 0x01;
	else
		argument &=0xFE;

	if(type == 1)//bulk or interrupt
		argument |= 0x02;
	else//control
		argument &= 0xFD;
	
	size <<= 2;
	size &= 0x3C;
	argument |= size;
	
	
	if(bEPPflags.bits.in_isr == 0)
		DISABLE;
	
	if(isSpi)
	{
		GPIOSetValue( 2, 0, 0 );//SS low
		outBuf[0]= command;
		outBuf[1]= argument;
		SSP_Send(SSP_NUM,outBuf,2);
		GPIOSetValue( 2, 0, 1 );//SS high
	}
	else
	{
		outportb(FT12X_COMMAND, command);
		outportb(FT12X_DATA, argument);
	}
	
	if(bEPPflags.bits.in_isr == 0)
		ENABLE;

}


